@@ -6,9 +6,10 @@
 // https://developers.google.com/open-source/licenses/bsd
 
 use crate::{
-    Mut, MutProxy, Proxied, SettableValue, View, ViewProxy,
-    __internal::Private,
-    __runtime::{InnerMap, InnerMapMut, RawMap, RawMapIter},
+    AsMut, AsView, IntoMut, IntoProxied, IntoView, MutProxied, MutProxy, Proxied, Proxy, View,
+    ViewProxy,
+    __internal::runtime::{InnerMap, InnerMapMut, RawMap, RawMapIter},
+    __internal::{Private, SealedInternal},
 };
 use std::marker::PhantomData;
 
@@ -59,12 +60,21 @@ impl<'msg, K: ?Sized, V: ?Sized> std::fmt::Debug for MapMut<'msg, K, V> {
     }
 }
 
-pub struct Map<K: ?Sized + Proxied, V: ?Sized + ProxiedInMapValue<K>> {
+pub struct Map<K: Proxied, V: ProxiedInMapValue<K>> {
     inner: InnerMap,
     _phantom: PhantomData<(PhantomData<K>, PhantomData<V>)>,
 }
 
-impl<K: ?Sized + Proxied, V: ?Sized + ProxiedInMapValue<K>> Drop for Map<K, V> {
+// SAFETY: `Map` is Sync because it does not implement interior mutability.
+unsafe impl<K: Proxied, V: ProxiedInMapValue<K>> Sync for Map<K, V> {}
+
+// SAFETY: `Map` is Send because it's not bound to a specific thread e.g.
+// it does not use thread-local data or similar.
+unsafe impl<K: Proxied, V: ProxiedInMapValue<K>> Send for Map<K, V> {}
+
+impl<K: Proxied, V: ProxiedInMapValue<K>> SealedInternal for Map<K, V> {}
+
+impl<K: Proxied, V: ProxiedInMapValue<K>> Drop for Map<K, V> {
     fn drop(&mut self) {
         // SAFETY:
         // - `drop` is only called once.
@@ -75,7 +85,7 @@ impl<K: ?Sized + Proxied, V: ?Sized + ProxiedInMapValue<K>> Drop for Map<K, V> {
 
 pub trait ProxiedInMapValue<K>: Proxied
 where
-    K: Proxied + ?Sized,
+    K: Proxied,
 {
     fn map_new(_private: Private) -> Map<K, Self>;
 
@@ -83,42 +93,62 @@ where
     /// - After `map_free`, no other methods on the input are safe to call.
     unsafe fn map_free(_private: Private, map: &mut Map<K, Self>);
 
-    fn map_clear(map: Mut<'_, Map<K, Self>>);
-    fn map_len(map: View<'_, Map<K, Self>>) -> usize;
-    fn map_insert(map: Mut<'_, Map<K, Self>>, key: View<'_, K>, value: View<'_, Self>) -> bool;
-    fn map_get<'a>(map: View<'a, Map<K, Self>>, key: View<'_, K>) -> Option<View<'a, Self>>;
-    fn map_remove(map: Mut<'_, Map<K, Self>>, key: View<'_, K>) -> bool;
+    fn map_clear(map: MapMut<K, Self>);
+    fn map_len(map: MapView<K, Self>) -> usize;
+    fn map_insert(map: MapMut<K, Self>, key: View<'_, K>, value: impl IntoProxied<Self>) -> bool;
+    fn map_get<'a>(map: MapView<'a, K, Self>, key: View<'_, K>) -> Option<View<'a, Self>>;
+    fn map_remove(map: MapMut<K, Self>, key: View<'_, K>) -> bool;
 
-    fn map_iter(map: View<'_, Map<K, Self>>) -> MapIter<'_, K, Self>;
+    fn map_iter(map: MapView<K, Self>) -> MapIter<K, Self>;
     fn map_iter_next<'a>(iter: &mut MapIter<'a, K, Self>) -> Option<(View<'a, K>, View<'a, Self>)>;
 }
 
-impl<K: Proxied + ?Sized, V: ProxiedInMapValue<K> + ?Sized> Proxied for Map<K, V> {
-    type View<'msg> = MapView<'msg, K, V> where K: 'msg, V: 'msg;
-    type Mut<'msg> = MapMut<'msg, K, V> where K: 'msg, V: 'msg;
+impl<K: Proxied, V: ProxiedInMapValue<K>> Proxied for Map<K, V> {
+    type View<'msg>
+        = MapView<'msg, K, V>
+    where
+        K: 'msg,
+        V: 'msg;
 }
 
-impl<'msg, K: Proxied + ?Sized, V: ProxiedInMapValue<K> + ?Sized> SettableValue<Map<K, V>>
-    for MapView<'msg, K, V>
-{
-    fn set_on<'b>(self, _private: Private, mut mutator: Mut<'b, Map<K, V>>)
-    where
-        Map<K, V>: 'b,
-    {
-        mutator.copy_from(self);
+impl<K: Proxied, V: ProxiedInMapValue<K>> AsView for Map<K, V> {
+    type Proxied = Self;
+
+    fn as_view(&self) -> MapView<'_, K, V> {
+        self.as_view()
     }
 }
 
-impl<'msg, K: Proxied + ?Sized, V: ProxiedInMapValue<K> + ?Sized> ViewProxy<'msg>
-    for MapView<'msg, K, V>
-{
+impl<K: Proxied, V: ProxiedInMapValue<K>> MutProxied for Map<K, V> {
+    type Mut<'msg>
+        = MapMut<'msg, K, V>
+    where
+        K: 'msg,
+        V: 'msg;
+}
+
+impl<K: Proxied, V: ProxiedInMapValue<K>> AsMut for Map<K, V> {
+    type MutProxied = Self;
+
+    fn as_mut(&mut self) -> MapMut<'_, K, V> {
+        self.as_mut()
+    }
+}
+
+impl<'msg, K: Proxied, V: ProxiedInMapValue<K>> SealedInternal for MapView<'msg, K, V> {}
+
+impl<'msg, K: Proxied, V: ProxiedInMapValue<K>> Proxy<'msg> for MapView<'msg, K, V> {}
+
+impl<'msg, K: Proxied, V: ProxiedInMapValue<K>> AsView for MapView<'msg, K, V> {
     type Proxied = Map<K, V>;
 
-    fn as_view(&self) -> View<'_, Self::Proxied> {
+    fn as_view(&self) -> MapView<'_, K, V> {
         *self
     }
+}
 
-    fn into_view<'shorter>(self) -> View<'shorter, Self::Proxied>
+impl<'msg, K: Proxied, V: ProxiedInMapValue<K>> IntoView<'msg> for MapView<'msg, K, V> {
+    fn into_view<'shorter>(self) -> MapView<'shorter, K, V>
     where
         'msg: 'shorter,
     {
@@ -126,16 +156,22 @@ impl<'msg, K: Proxied + ?Sized, V: ProxiedInMapValue<K> + ?Sized> ViewProxy<'msg
     }
 }
 
-impl<'msg, K: Proxied + ?Sized, V: ProxiedInMapValue<K> + ?Sized> ViewProxy<'msg>
-    for MapMut<'msg, K, V>
-{
+impl<'msg, K: Proxied, V: ProxiedInMapValue<K>> ViewProxy<'msg> for MapView<'msg, K, V> {}
+
+impl<'msg, K: Proxied, V: ProxiedInMapValue<K>> SealedInternal for MapMut<'msg, K, V> {}
+
+impl<'msg, K: Proxied, V: ProxiedInMapValue<K>> Proxy<'msg> for MapMut<'msg, K, V> {}
+
+impl<'msg, K: Proxied, V: ProxiedInMapValue<K>> AsView for MapMut<'msg, K, V> {
     type Proxied = Map<K, V>;
 
-    fn as_view(&self) -> View<'_, Self::Proxied> {
+    fn as_view(&self) -> MapView<'_, K, V> {
         MapView { raw: self.inner.raw, _phantom: PhantomData }
     }
+}
 
-    fn into_view<'shorter>(self) -> View<'shorter, Self::Proxied>
+impl<'msg, K: Proxied, V: ProxiedInMapValue<K>> IntoView<'msg> for MapMut<'msg, K, V> {
+    fn into_view<'shorter>(self) -> MapView<'shorter, K, V>
     where
         'msg: 'shorter,
     {
@@ -143,14 +179,16 @@ impl<'msg, K: Proxied + ?Sized, V: ProxiedInMapValue<K> + ?Sized> ViewProxy<'msg
     }
 }
 
-impl<'msg, K: Proxied + ?Sized, V: ProxiedInMapValue<K> + ?Sized> MutProxy<'msg>
-    for MapMut<'msg, K, V>
-{
-    fn as_mut(&mut self) -> Mut<'_, Self::Proxied> {
+impl<'msg, K: Proxied, V: ProxiedInMapValue<K>> AsMut for MapMut<'msg, K, V> {
+    type MutProxied = Map<K, V>;
+
+    fn as_mut(&mut self) -> MapMut<'_, K, V> {
         MapMut { inner: self.inner, _phantom: PhantomData }
     }
+}
 
-    fn into_mut<'shorter>(self) -> Mut<'shorter, Self::Proxied>
+impl<'msg, K: Proxied, V: ProxiedInMapValue<K>> IntoMut<'msg> for MapMut<'msg, K, V> {
+    fn into_mut<'shorter>(self) -> MapMut<'shorter, K, V>
     where
         'msg: 'shorter,
     {
@@ -158,10 +196,12 @@ impl<'msg, K: Proxied + ?Sized, V: ProxiedInMapValue<K> + ?Sized> MutProxy<'msg>
     }
 }
 
+impl<'msg, K: Proxied, V: ProxiedInMapValue<K>> MutProxy<'msg> for MapMut<'msg, K, V> {}
+
 impl<K, V> Map<K, V>
 where
-    K: Proxied + ?Sized,
-    V: ProxiedInMapValue<K> + ?Sized,
+    K: Proxied,
+    V: ProxiedInMapValue<K>,
 {
     pub fn new() -> Self {
         V::map_new(Private)
@@ -187,8 +227,8 @@ where
 
 impl<K, V> Default for Map<K, V>
 where
-    K: Proxied + ?Sized,
-    V: ProxiedInMapValue<K> + ?Sized,
+    K: Proxied,
+    V: ProxiedInMapValue<K>,
 {
     fn default() -> Self {
         Map::new()
@@ -212,8 +252,8 @@ impl<'msg, K: ?Sized, V: ?Sized> MapView<'msg, K, V> {
 
 impl<'msg, K, V> MapView<'msg, K, V>
 where
-    K: Proxied + ?Sized + 'msg,
-    V: ProxiedInMapValue<K> + ?Sized + 'msg,
+    K: Proxied + 'msg,
+    V: ProxiedInMapValue<K> + 'msg,
 {
     pub fn get<'a>(self, key: impl Into<View<'a, K>>) -> Option<View<'msg, V>>
     where
@@ -270,8 +310,8 @@ impl<'msg, K: ?Sized, V: ?Sized> MapMut<'msg, K, V> {
 
 impl<'msg, K, V> MapMut<'msg, K, V>
 where
-    K: Proxied + ?Sized + 'msg,
-    V: ProxiedInMapValue<K> + ?Sized + 'msg,
+    K: Proxied + 'msg,
+    V: ProxiedInMapValue<K> + 'msg,
 {
     pub fn len(&self) -> usize {
         self.as_view().len()
@@ -284,16 +324,11 @@ where
     /// Adds a key-value pair to the map.
     ///
     /// Returns `true` if the entry was newly inserted.
-    pub fn insert<'a, 'b>(
-        &mut self,
-        key: impl Into<View<'a, K>>,
-        value: impl Into<View<'b, V>>,
-    ) -> bool
+    pub fn insert<'a>(&mut self, key: impl Into<View<'a, K>>, value: impl IntoProxied<V>) -> bool
     where
         K: 'a,
-        V: 'b,
     {
-        V::map_insert(self.as_mut(), key.into(), value.into())
+        V::map_insert(self.as_mut(), key.into(), value)
     }
 
     pub fn remove<'a>(&mut self, key: impl Into<View<'a, K>>) -> bool
@@ -314,12 +349,11 @@ where
         V::map_get(self.as_view(), key.into())
     }
 
-    pub fn copy_from<'a, 'b>(
+    pub fn copy_from<'a>(
         &mut self,
-        src: impl IntoIterator<Item = (impl Into<View<'a, K>>, impl Into<View<'b, V>>)>,
+        src: impl IntoIterator<Item = (impl Into<View<'a, K>>, impl IntoProxied<V>)>,
     ) where
         K: 'a,
-        V: 'b,
     {
         //TODO
         self.clear();
@@ -331,7 +365,7 @@ where
     /// Returns an iterator visiting all key-value pairs in arbitrary order.
     ///
     /// The iterator element type is `(View<K>, View<V>)`.
-    pub fn iter(&self) -> MapIter<'_, K, V> {
+    pub fn iter(&self) -> MapIter<K, V> {
         self.into_iter()
     }
 
@@ -350,6 +384,45 @@ where
     }
 }
 
+impl<'msg, K, V> IntoProxied<Map<K, V>> for MapView<'msg, K, V>
+where
+    K: Proxied,
+    V: ProxiedInMapValue<K>,
+    View<'msg, V>: IntoProxied<V>,
+{
+    fn into_proxied(self, _private: Private) -> Map<K, V> {
+        let mut m = Map::<K, V>::new();
+        m.as_mut().copy_from(self);
+        m
+    }
+}
+
+impl<'msg, K, V> IntoProxied<Map<K, V>> for MapMut<'msg, K, V>
+where
+    K: Proxied,
+    V: ProxiedInMapValue<K>,
+    View<'msg, V>: IntoProxied<V>,
+{
+    fn into_proxied(self, _private: Private) -> Map<K, V> {
+        self.into_view().into_proxied(Private)
+    }
+}
+
+impl<'msg, 'k, 'v, K, KView, V, VView, I> IntoProxied<Map<K, V>> for I
+where
+    I: Iterator<Item = (KView, VView)>,
+    K: Proxied + 'msg + 'k,
+    V: ProxiedInMapValue<K> + 'msg + 'v,
+    KView: Into<View<'k, K>>,
+    VView: IntoProxied<V>,
+{
+    fn into_proxied(self, _private: Private) -> Map<K, V> {
+        let mut m = Map::<K, V>::new();
+        m.as_mut().extend(self);
+        m
+    }
+}
+
 /// An iterator visiting all key-value pairs in arbitrary order.
 ///
 /// The iterator element type is `(View<Key>, View<Value>)`.
@@ -376,8 +449,8 @@ impl<'msg, K: ?Sized, V: ?Sized> MapIter<'msg, K, V> {
 
 impl<'msg, K, V> Iterator for MapIter<'msg, K, V>
 where
-    K: Proxied + ?Sized + 'msg,
-    V: ProxiedInMapValue<K> + ?Sized + 'msg,
+    K: Proxied + 'msg,
+    V: ProxiedInMapValue<K> + 'msg,
 {
     type Item = (View<'msg, K>, View<'msg, V>);
 
@@ -388,8 +461,8 @@ where
 
 impl<'msg, K, V> IntoIterator for MapView<'msg, K, V>
 where
-    K: Proxied + ?Sized + 'msg,
-    V: ProxiedInMapValue<K> + ?Sized + 'msg,
+    K: Proxied + 'msg,
+    V: ProxiedInMapValue<K> + 'msg,
 {
     type IntoIter = MapIter<'msg, K, V>;
     type Item = (View<'msg, K>, View<'msg, V>);
@@ -401,8 +474,8 @@ where
 
 impl<'msg, K, V> IntoIterator for &'msg Map<K, V>
 where
-    K: Proxied + ?Sized + 'msg,
-    V: ProxiedInMapValue<K> + ?Sized + 'msg,
+    K: Proxied + 'msg,
+    V: ProxiedInMapValue<K> + 'msg,
 {
     type IntoIter = MapIter<'msg, K, V>;
     type Item = (View<'msg, K>, View<'msg, V>);
@@ -415,8 +488,8 @@ where
 impl<'a, 'msg, K, V> IntoIterator for &'a MapView<'msg, K, V>
 where
     'msg: 'a,
-    K: Proxied + ?Sized + 'msg,
-    V: ProxiedInMapValue<K> + ?Sized + 'msg,
+    K: Proxied + 'msg,
+    V: ProxiedInMapValue<K> + 'msg,
 {
     type IntoIter = MapIter<'msg, K, V>;
     type Item = (View<'msg, K>, View<'msg, V>);
@@ -429,8 +502,8 @@ where
 impl<'a, 'msg, K, V> IntoIterator for &'a MapMut<'msg, K, V>
 where
     'msg: 'a,
-    K: Proxied + ?Sized + 'msg,
-    V: ProxiedInMapValue<K> + ?Sized + 'msg,
+    K: Proxied + 'msg,
+    V: ProxiedInMapValue<K> + 'msg,
 {
     type IntoIter = MapIter<'a, K, V>;
     // The View's are valid for 'a instead of 'msg.
@@ -444,10 +517,10 @@ where
 
 impl<'msg, 'k, 'v, KView, VView, K, V> Extend<(KView, VView)> for MapMut<'msg, K, V>
 where
-    K: Proxied + ?Sized + 'msg + 'k,
-    V: ProxiedInMapValue<K> + ?Sized + 'msg + 'v,
+    K: Proxied + 'msg + 'k,
+    V: ProxiedInMapValue<K> + 'msg + 'v,
     KView: Into<View<'k, K>>,
-    VView: Into<View<'v, V>>,
+    VView: IntoProxied<V>,
 {
     fn extend<T: IntoIterator<Item = (KView, VView)>>(&mut self, iter: T) {
         for (k, v) in iter.into_iter() {
@@ -459,10 +532,10 @@ where
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::ProtoStr;
+    use crate::{ProtoBytes, ProtoStr, ProtoString};
     use googletest::prelude::*;
 
-    #[test]
+    #[gtest]
     fn test_proxied_scalar() {
         let mut map: Map<i32, i64> = Map::new();
         let mut map_mut = map.as_mut();
@@ -488,9 +561,9 @@ mod tests {
         assert_that!(map_view_4.is_empty(), eq(false));
     }
 
-    #[test]
+    #[gtest]
     fn test_proxied_str() {
-        let mut map: Map<ProtoStr, ProtoStr> = Map::new();
+        let mut map: Map<ProtoString, ProtoString> = Map::new();
         let mut map_mut = map.as_mut();
         map_mut.insert("a", "b");
 
@@ -513,9 +586,9 @@ mod tests {
         assert_that!(map_view_4.is_empty(), eq(false));
     }
 
-    #[test]
+    #[gtest]
     fn test_proxied_iter() {
-        let mut map: Map<i32, ProtoStr> = Map::new();
+        let mut map: Map<i32, ProtoString> = Map::new();
         let mut map_mut = map.as_mut();
         map_mut.insert(15, "fizzbuzz");
         map_mut.insert(5, "buzz");
@@ -529,13 +602,13 @@ mod tests {
         assert_that!(
             map.as_view().iter().collect::<Vec<_>>(),
             unordered_elements_are![
-                eq((3, ProtoStr::from_str("fizz"))),
-                eq((5, ProtoStr::from_str("buzz"))),
-                eq((15, ProtoStr::from_str("fizzbuzz")))
+                eq(&(3, ProtoStr::from_str("fizz"))),
+                eq(&(5, ProtoStr::from_str("buzz"))),
+                eq(&(15, ProtoStr::from_str("fizzbuzz")))
             ]
         );
         assert_that!(
-            map.as_view().into_iter().collect::<Vec<_>>(),
+            map.as_view(),
             unordered_elements_are![
                 eq((3, ProtoStr::from_str("fizz"))),
                 eq((5, ProtoStr::from_str("buzz"))),
@@ -543,7 +616,15 @@ mod tests {
             ]
         );
         assert_that!(
-            map.as_view().into_iter().collect::<Vec<_>>(),
+            map.as_mut().iter().collect::<Vec<_>>(),
+            unordered_elements_are![
+                eq(&(3, ProtoStr::from_str("fizz"))),
+                eq(&(5, ProtoStr::from_str("buzz"))),
+                eq(&(15, ProtoStr::from_str("fizzbuzz")))
+            ]
+        );
+        assert_that!(
+            map.as_mut(),
             unordered_elements_are![
                 eq((3, ProtoStr::from_str("fizz"))),
                 eq((5, ProtoStr::from_str("buzz"))),
@@ -552,22 +633,19 @@ mod tests {
         );
     }
 
-    #[test]
+    #[gtest]
     fn test_overwrite_insert() {
-        let mut map: Map<i32, ProtoStr> = Map::new();
+        let mut map: Map<i32, ProtoString> = Map::new();
         let mut map_mut = map.as_mut();
         assert!(map_mut.insert(0, "fizz"));
         // insert should return false when the key is already present
         assert!(!map_mut.insert(0, "buzz"));
-        assert_that!(
-            map.as_mut().iter().collect::<Vec<_>>(),
-            unordered_elements_are![eq((0, ProtoStr::from_str("buzz"))),]
-        );
+        assert_that!(map.as_mut(), unordered_elements_are![eq((0, ProtoStr::from_str("buzz"))),]);
     }
 
-    #[test]
+    #[gtest]
     fn test_extend() {
-        let mut map: Map<i32, ProtoStr> = Map::new();
+        let mut map: Map<i32, ProtoString> = Map::new();
         let mut map_mut = map.as_mut();
 
         map_mut.extend([(0, ""); 0]);
@@ -576,7 +654,7 @@ mod tests {
         map_mut.extend([(0, "fizz"), (1, "buzz"), (2, "fizzbuzz")]);
 
         assert_that!(
-            map.as_view().into_iter().collect::<Vec<_>>(),
+            map.as_view(),
             unordered_elements_are![
                 eq((0, ProtoStr::from_str("fizz"))),
                 eq((1, ProtoStr::from_str("buzz"))),
@@ -584,7 +662,7 @@ mod tests {
             ]
         );
 
-        let mut map_2: Map<i32, ProtoStr> = Map::new();
+        let mut map_2: Map<i32, ProtoString> = Map::new();
         let mut map_2_mut = map_2.as_mut();
         map_2_mut.extend([(2, "bing"), (3, "bong")]);
 
@@ -592,7 +670,7 @@ mod tests {
         map_mut.extend(&map_2);
 
         assert_that!(
-            map.as_view().into_iter().collect::<Vec<_>>(),
+            map.as_view(),
             unordered_elements_are![
                 eq((0, ProtoStr::from_str("fizz"))),
                 eq((1, ProtoStr::from_str("buzz"))),
@@ -602,14 +680,14 @@ mod tests {
         );
     }
 
-    #[test]
+    #[gtest]
     fn test_copy_from() {
-        let mut map: Map<i32, ProtoStr> = Map::new();
+        let mut map: Map<i32, ProtoString> = Map::new();
         let mut map_mut = map.as_mut();
         map_mut.copy_from([(0, "fizz"), (1, "buzz"), (2, "fizzbuzz")]);
 
         assert_that!(
-            map.as_view().into_iter().collect::<Vec<_>>(),
+            map.as_view(),
             unordered_elements_are![
                 eq((0, ProtoStr::from_str("fizz"))),
                 eq((1, ProtoStr::from_str("buzz"))),
@@ -617,7 +695,7 @@ mod tests {
             ]
         );
 
-        let mut map_2: Map<i32, ProtoStr> = Map::new();
+        let mut map_2: Map<i32, ProtoString> = Map::new();
         let mut map_2_mut = map_2.as_mut();
         map_2_mut.copy_from([(2, "bing"), (3, "bong")]);
 
@@ -625,7 +703,7 @@ mod tests {
         map_mut.copy_from(&map_2);
 
         assert_that!(
-            map.as_view().into_iter().collect::<Vec<_>>(),
+            map.as_view(),
             unordered_elements_are![
                 eq((2, ProtoStr::from_str("bing"))),
                 eq((3, ProtoStr::from_str("bong")))
@@ -633,7 +711,7 @@ mod tests {
         );
     }
 
-    #[test]
+    #[gtest]
     fn test_all_maps_can_be_constructed() {
         macro_rules! gen_proto_values {
             ($key_t:ty, $($value_t:ty),*) => {
@@ -647,15 +725,15 @@ mod tests {
         macro_rules! gen_proto_keys {
             ($($key_t:ty),*) => {
                 $(
-                    gen_proto_values!($key_t, f32, f64, i32, u32, i64, bool, ProtoStr, [u8]);
+                    gen_proto_values!($key_t, f32, f64, i32, u32, i64, bool, ProtoString, ProtoBytes);
                 )*
             }
         }
 
-        gen_proto_keys!(i32, u32, i64, u64, bool, ProtoStr);
+        gen_proto_keys!(i32, u32, i64, u64, bool, ProtoString);
     }
 
-    #[test]
+    #[gtest]
     fn test_dbg() {
         let mut map = Map::<i32, f64>::new();
         assert_that!(format!("{:?}", map.as_view()), eq("MapView(\"i32\", \"f64\")"));
