@@ -39,7 +39,7 @@ void SetCordVariables(
       "\"", absl::CEscape(descriptor->default_value_string()), "\"");
   (*variables)["default_length"] =
       absl::StrCat(descriptor->default_value_string().length());
-  (*variables)["full_name"] = descriptor->full_name();
+  (*variables)["full_name"] = std::string(descriptor->full_name());
   // For one of Cords
   (*variables)["default_variable_name"] = MakeDefaultName(descriptor);
   (*variables)["default_variable_field"] = MakeDefaultFieldName(descriptor);
@@ -103,7 +103,7 @@ class CordFieldGenerator : public FieldGeneratorBase {
   void GenerateOneofCopyConstruct(io::Printer* p) const override {
     auto vars = p->WithVars(variables_);
     p->Emit(R"cc(
-      $field$ = ::$proto_ns$::Arena::Create<absl::Cord>(arena, *from.$field$);
+      $field$ = $pb$::Arena::Create<absl::Cord>(arena, *from.$field$);
     )cc");
   }
 };
@@ -165,7 +165,7 @@ void CordFieldGenerator::GenerateAccessorDeclarations(
       "private:\n"
       "const ::absl::Cord& ${1$_internal_$name$$}$() const;\n"
       "void ${1$_internal_set_$name$$}$(const ::absl::Cord& value);\n"
-      "::absl::Cord* ${1$_internal_mutable_$name$$}$();\n"
+      "::absl::Cord* $nonnull$ ${1$_internal_mutable_$name$$}$();\n"
       "public:\n",
       field_);
 }
@@ -214,7 +214,8 @@ void CordFieldGenerator::GenerateInlineAccessorDefinitions(
     }
   )cc");
   printer->Emit(R"cc(
-    inline ::absl::Cord* $classname$::_internal_mutable_$name_internal$() {
+    inline ::absl::Cord* $nonnull$
+    $classname$::_internal_mutable_$name_internal$() {
       $set_hasbit$;
       return &$field$;
     }
@@ -275,7 +276,7 @@ void CordFieldGenerator::GenerateByteSize(io::Printer* printer) const {
   Formatter format(printer, variables_);
   format(
       "total_size += $tag_size$ +\n"
-      "  ::$proto_ns$::internal::WireFormatLite::$declared_type$Size(\n"
+      "  $pbi$::WireFormatLite::$declared_type$Size(\n"
       "    this_._internal_$name$());\n");
 }
 
@@ -317,7 +318,7 @@ CordOneofFieldGenerator::CordOneofFieldGenerator(
 void CordOneofFieldGenerator::GeneratePrivateMembers(
     io::Printer* printer) const {
   Formatter format(printer, variables_);
-  format("::absl::Cord *$name$_;\n");
+  format("::absl::Cord* $nonnull$ $name$_;\n");
 }
 
 void CordOneofFieldGenerator::GenerateStaticMembers(
@@ -361,7 +362,7 @@ void CordOneofFieldGenerator::GenerateInlineAccessorDefinitions(
         clear_$oneof_name$();
         set_has_$name_internal$();
         $field$ = new ::absl::Cord;
-        ::$proto_ns$::Arena* arena = GetArena();
+        $pb$::Arena* arena = GetArena();
         if (arena != nullptr) {
           arena->Own($field$);
         }
@@ -378,7 +379,7 @@ void CordOneofFieldGenerator::GenerateInlineAccessorDefinitions(
         clear_$oneof_name$();
         set_has_$name_internal$();
         $field$ = new ::absl::Cord;
-        ::$proto_ns$::Arena* arena = GetArena();
+        $pb$::Arena* arena = GetArena();
         if (arena != nullptr) {
           arena->Own($field$);
         }
@@ -389,12 +390,13 @@ void CordOneofFieldGenerator::GenerateInlineAccessorDefinitions(
     }
   )cc");
   printer->Emit(R"cc(
-    inline ::absl::Cord* $classname$::_internal_mutable_$name_internal$() {
+    inline ::absl::Cord* $nonnull$
+    $classname$::_internal_mutable_$name_internal$() {
       if ($not_has_field$) {
         clear_$oneof_name$();
         set_has_$name_internal$();
         $field$ = new ::absl::Cord;
-        ::$proto_ns$::Arena* arena = GetArena();
+        $pb$::Arena* arena = GetArena();
         if (arena != nullptr) {
           arena->Own($field$);
         }
@@ -445,7 +447,7 @@ void CordOneofFieldGenerator::GenerateArenaDestructorCode(
 void CordOneofFieldGenerator::GenerateMergingCode(io::Printer* printer) const {
   printer->Emit(R"cc(
     if (oneof_needs_init) {
-      _this->$field$ = ::$proto_ns$::Arena::Create<absl::Cord>(arena);
+      _this->$field$ = $pb$::Arena::Create<absl::Cord>(arena);
     }
     *_this->$field$ = *from.$field$;
   )cc");
