@@ -7,6 +7,8 @@
 
 #include "google/protobuf/compiler/python/pyi_generator.h"
 
+#include <cstddef>
+#include <memory>
 #include <string>
 #include <utility>
 #include <vector>
@@ -16,8 +18,10 @@
 #include "absl/log/absl_log.h"
 #include "absl/strings/ascii.h"
 #include "absl/strings/match.h"
+#include "absl/strings/str_cat.h"
 #include "absl/strings/str_split.h"
 #include "absl/strings/string_view.h"
+#include "absl/synchronization/mutex.h"
 #include "google/protobuf/compiler/code_generator.h"
 #include "google/protobuf/compiler/python/helpers.h"
 #include "google/protobuf/descriptor.h"
@@ -32,14 +36,14 @@ namespace python {
 
 PyiGenerator::PyiGenerator() : file_(nullptr) {}
 
-PyiGenerator::~PyiGenerator() {}
+PyiGenerator::~PyiGenerator() = default;
 
 template <typename DescriptorT>
 std::string PyiGenerator::ModuleLevelName(const DescriptorT& descriptor) const {
   std::string name = NamePrefixedWithNestedTypes(descriptor, ".");
   if (descriptor.file() != file_) {
     std::string module_alias;
-    std::string filename = descriptor.file()->name();
+    const absl::string_view filename = descriptor.file()->name();
     if (import_map_.find(filename) == import_map_.end()) {
       std::string module_name = ModuleName(descriptor.file()->name());
       std::vector<absl::string_view> tokens = absl::StrSplit(module_name, '.');
@@ -60,18 +64,19 @@ std::string PyiGenerator::InternalPackage() const {
 
 struct ImportModules {
   bool has_repeated = false;    // _containers
-  bool has_iterable = false;    // typing.Iterable
+  bool has_iterable = false;    // collections.abc.Iterable
   bool has_messages = false;    // _message
   bool has_enums = false;       // _enum_type_wrapper
   bool has_extendable = false;  // _python_message
-  bool has_mapping = false;     // typing.Mapping
+  bool has_mapping = false;     // collections.abc.Mapping
   bool has_optional = false;    // typing.Optional
   bool has_union = false;       // typing.Union
+  bool has_callable = false;    // typing.Callable
   bool has_well_known_type = false;
 };
 
 // Checks whether a descriptor name matches a well-known type.
-bool IsWellKnownType(const std::string& name) {
+bool IsWellKnownType(const absl::string_view name) {
   // LINT.IfChange(wktbases)
   return (name == "google.protobuf.Any" ||
           name == "google.protobuf.Duration" ||
@@ -132,7 +137,7 @@ void CheckImportModules(const Descriptor* descriptor,
 void PyiGenerator::PrintImportForDescriptor(
     const FileDescriptor& desc, absl::flat_hash_set<std::string>* seen_aliases,
     bool* has_importlib) const {
-  const std::string& filename = desc.name();
+  const absl::string_view filename = desc.name();
   std::string module_name_owned = StrippedModuleName(filename);
   absl::string_view module_name(module_name_owned);
   size_t last_dot_pos = module_name.rfind('.');
@@ -189,6 +194,7 @@ void PyiGenerator::PrintImports() const {
     import_modules.has_enums = true;
   }
   if (!opensource_runtime_ && file_->service_count() > 0) {
+    import_modules.has_callable = true;
     import_modules.has_optional = true;
     import_modules.has_union = true;
   }
@@ -240,17 +246,27 @@ void PyiGenerator::PrintImports() const {
           "from google3.net.rpc.python import rpcserver as _rpcserver\n");
     }
   }
+  if (import_modules.has_iterable || import_modules.has_mapping) {
+    printer_->Print("from collections.abc import");
+    if (import_modules.has_iterable) {
+      printer_->Print(" Iterable as _Iterable");
+      if (import_modules.has_mapping) {
+        printer_->Print(",");
+      }
+    }
+    if (import_modules.has_mapping) {
+      printer_->Print(" Mapping as _Mapping");
+    }
+    printer_->Print("\n");
+  }
   printer_->Print("from typing import ");
   if (!opensource_runtime_ && file_->service_count() > 0) {
     printer_->Print("Any as _Any, ");
   }
+  if (import_modules.has_callable) {
+    printer_->Print("Callable as _Callable, ");
+  }
   printer_->Print("ClassVar as _ClassVar");
-  if (import_modules.has_iterable) {
-    printer_->Print(", Iterable as _Iterable");
-  }
-  if (import_modules.has_mapping) {
-    printer_->Print(", Mapping as _Mapping");
-  }
   if (import_modules.has_optional) {
     printer_->Print(", Optional as _Optional");
   }
@@ -276,7 +292,7 @@ void PyiGenerator::PrintImports() const {
                       public_dep->enum_type(i)->name());
     }
   }
-printer_->Print("\n");
+  printer_->Print("\n");
 }
 
 // Annotate wrapper for debugging purposes
@@ -284,11 +300,11 @@ printer_->Print("\n");
 template <typename DescriptorT>
 void PyiGenerator::Annotate(const std::string& label,
                             const DescriptorT* descriptor) const {
-printer_->Annotate(label.c_str(), descriptor);
+  printer_->Annotate(label, descriptor);
 }
 
 void PyiGenerator::PrintEnum(const EnumDescriptor& enum_descriptor) const {
-  std::string enum_name = enum_descriptor.name();
+  const absl::string_view enum_name = enum_descriptor.name();
   printer_->Print(
       "class $enum_name$(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):\n"
       "    __slots__ = ()\n",
@@ -385,7 +401,7 @@ void PyiGenerator::PrintMessage(
   if (!is_nested) {
     printer_->Print("\n");
   }
-  std::string class_name = message_descriptor.name();
+  const absl::string_view class_name = message_descriptor.name();
   std::string extra_base;
   // A well-known type needs to inherit from its corresponding base class in
   // net/proto2/python/internal/well_known_types.
@@ -475,23 +491,25 @@ void PyiGenerator::PrintMessage(
   }
 
   // Prints __init__
-  printer_->Print("def __init__(self");
-  bool has_key_words = false;
-  bool is_first = true;
+  printer_->Print("def __init__(");
+  // If the message has a field named "self" (see b/144146793), it can still be
+  // passed to the initializer, which takes those as **kwargs. To avoid name
+  // collision, we rename the self parameter by appending underscores until it
+  // no longer collides. The self-parameter is in fact positional-only, so the
+  // name in the pyi doesn't matter with regard to what runtime usage is valid.
+  std::string self_arg_name = "self";
+  while (message_descriptor.FindFieldByName(self_arg_name) != nullptr) {
+    self_arg_name.append("_");
+  }
+  printer_->Print(self_arg_name);
+  bool has_python_keywords = false;
   for (int i = 0; i < message_descriptor.field_count(); ++i) {
     const FieldDescriptor* field_des = message_descriptor.field(i);
     if (IsPythonKeyword(field_des->name())) {
-      has_key_words = true;
+      has_python_keywords = true;
       continue;
     }
-    std::string field_name = field_des->name();
-    if (is_first && field_name == "self") {
-      // See b/144146793 for an example of real code that generates a (self,
-      // self) method signature. Since repeating a parameter name is illegal in
-      // Python, we rename the duplicate self.
-      field_name = "self_";
-    }
-    is_first = false;
+    std::string field_name = std::string(field_des->name());
     printer_->Print(", $field_name$: ", "field_name", field_name);
     Annotate("field_name", field_des);
     if (field_des->is_repeated() ||
@@ -533,7 +551,7 @@ void PyiGenerator::PrintMessage(
     }
     printer_->Print(" = ...");
   }
-  if (has_key_words) {
+  if (has_python_keywords) {
     printer_->Print(", **kwargs");
   }
   printer_->Print(") -> None: ...\n");
@@ -567,7 +585,7 @@ bool PyiGenerator::Generate(const FileDescriptor* file,
   import_map_.clear();
   // Calculate file name.
   file_ = file;
-  // In google3, devtools/python/blaze/pytype/pytype_impl.bzl uses --pyi_out to
+  // In google3, devtools/python/bazel/pytype/pytype_impl.bzl uses --pyi_out to
   // directly set the output file name.
   std::vector<std::pair<std::string, std::string> > options;
   ParseGeneratorParameter(parameter, &options);
