@@ -8,15 +8,16 @@
 #include "google/protobuf/generated_message_bases.h"
 
 #include <cstddef>
+#include <cstdint>
 
+#include "absl/base/optimization.h"
+#include "absl/log/absl_check.h"
 #include "google/protobuf/generated_message_reflection.h"
 #include "google/protobuf/io/coded_stream.h"
-#include "google/protobuf/io/zero_copy_stream_impl.h"
 #include "google/protobuf/message_lite.h"
 #include "google/protobuf/parse_context.h"
 #include "google/protobuf/unknown_field_set.h"
 #include "google/protobuf/wire_format.h"
-#include "google/protobuf/wire_format_lite.h"
 
 // Must be last:
 #include "google/protobuf/port_def.inc"
@@ -37,6 +38,11 @@ ZeroFieldsBase::~ZeroFieldsBase() {
   _internal_metadata_.Delete<UnknownFieldSet>();
 }
 
+void ZeroFieldsBase::SharedDtor(MessageLite& msg) {
+  static_cast<ZeroFieldsBase&>(msg)
+      ._internal_metadata_.Delete<UnknownFieldSet>();
+}
+
 size_t ZeroFieldsBase::ByteSizeLong(const MessageLite& base) {
   auto& msg = static_cast<const ZeroFieldsBase&>(base);
   return msg.MaybeComputeUnknownFieldsSize(0, &msg._impl_._cached_size_);
@@ -46,7 +52,7 @@ size_t ZeroFieldsBase::ByteSizeLong(const MessageLite& base) {
                                               ::uint8_t* target,
                                               io::EpsCopyOutputStream* stream) {
   auto& this_ = static_cast<const ZeroFieldsBase&>(msg);
-  if (PROTOBUF_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
+  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
     target = internal::WireFormat::InternalSerializeUnknownFieldsToArray(
         this_._internal_metadata_.unknown_fields<UnknownFieldSet>(
             UnknownFieldSet::default_instance),
